/** File Header		***************************************************************************
*** EXTENSION: 			C
*** DESCRIPTION: 		DMA functions.
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	11 JAN 1999
*** VERSION: 1.0    ***************************************************************************/

#include "DMA.h"

/** Function Header	***************************************************************************
***	DESCRIPTION:		Writes a command to a DMA channel,
***						see DMA.h for the command options and use them as follow:
***						UINT command = FLY_BY | DTL_8BYTES | CHANNEL_ENABLE...
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	11 JAN 1999
*** VERSION: 1.0    ***************************************************************************/

void dma_command(UINT channel,UINT command)
{
	if(channel > 3 ) return;
	write_internal_reg(CHANNEL0CONTROL + channel*4,command);
}


/** Function Header ***************************************************************************
***	DESCRIPTION:		Performs a DMA transfer.
***						see DMA.h for the command options and use them as follow:
***						UINT command = FLY_BY | DTL_8BYTES | ALIGN_TOWARD_DEST... (no need for CHANNEL_ENABLE)
***						When return_flag = true the function returns immediately, otherwise it waits for
***						the transfer to end.
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	11 JAN 1999
*** VERSION: 1.0    ***************************************************************************/

void perform_dma(UINT channel,UINT source_addr,UINT dest_addr,UINT num_of_bytes,UINT command,Boolean return_flag)
{
	if(channel > 3 ) return;

    /* Wait if the channel is busy      */
	while(is_dma_activ(channel));
	
    write_internal_reg(CHANNEL0_DMA_BYTE_COUNT + channel*4,num_of_bytes);
	write_internal_reg(CHANNEL0_DMA_SOURCE_ADDRESS + channel*4,source_addr);
	write_internal_reg(CHANNEL0_DMA_DESTINATION_ADDRESS + channel*4,dest_addr);
	write_internal_reg(CHANNEL0NEXT_RECORD_POINTER + channel*4,NULL);
	command = command | CHANNEL_ENABLE | NON_CHAIN_MOD;
	write_internal_reg(CHANNEL0CONTROL + channel*4,command);
	if(return_flag) return;
	while(is_dma_activ(channel));
}

/** Function Header ***************************************************************************
***	DESCRIPTION:		Performs a DMA chain mode transfer.
***						see DMA.h for the command options and use them as follow:
***						UINT command = FLY_BY | DTL_8BYTES | ALIGN_TOWARD_DEST... (no need for CHANNEL_ENABLE)
***						When return_flag = RETURN_IMEDIATLY the function returns immediately, otherwise it waits for
***						the transfer to end, if next_recored_pointer = NULL then this function will
***                     act as perform_dma, DMA_RECORED structures must be filled before calling
***                     the function.
***                     Attention: If we need an endless DMA transfer the linked list`s tail must
***                     point to the head!!!
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	30 June 1999
*** VERSION: 1.0    ***************************************************************************/

/** Usage exsample  ***************************************************************************
    
    DMA_RECORED dma_recored_array[3];
    // Copy a 1/4 mega of data using chain mode DMA.
    dma_recored_array[0].ByteCnt = _64KB;
    dma_recored_array[0].DestAdd = dest_address + _64KB;
    dma_recored_array[0].SrcAdd  = source_address + _64KB;
    dma_recored_array[0].NextRecPtr = &dma_recored_array[1];
    dma_recored_array[1].ByteCnt = _64KB;
    dma_recored_array[1].DestAdd = dest_address + 2*_64KB;
    dma_recored_array[1].SrcAdd  = source_address + 2*_64KB;
    dma_recored_array[1].NextRecPtr = &dma_recored_array[2];
    dma_recored_array[2].ByteCnt = _64KB;
    dma_recored_array[2].DestAdd = dest_address + 3*_64KB;
    dma_recored_array[2].SrcAdd  = source_address + 3*_64KB;
    dma_recored_array[2].NextRecPtr = NULL;
    
    perform_cm_dma(0,source_address,dest_address,_64KB,PLAIN,WAIT_TO_END,&dma_recored_array[0]);
    
*** End of Usage exsample  *******************************************************************/

void perform_cm_dma(UINT channel,UINT source_addr,UINT dest_addr,UINT num_of_bytes,
                       UINT command,Boolean return_flag,DMA_RECORED * next_recored_pointer)
{
    UINT temp_data;
    DMA_RECORED * next = next_recored_pointer;
    if(channel > 3 ) return;
    
    /* Wait if the channel is busy  */
    while(is_dma_activ(channel));
    while(next != NULL)
    {
        write_word((UINT)next     ,next->ByteCnt);
        temp_data = (UINT)next->SrcAdd;
        write_word((UINT)next + 4 ,temp_data & 0x5fffffff);
        temp_data = (UINT)next->DestAdd;
        write_word((UINT)next + 8 ,temp_data & 0x5fffffff);
        temp_data = (UINT)next->NextRecPtr;
        write_word((UINT)next + 12,temp_data & 0x5fffffff);
        next = (DMA_RECORED *)temp_data;
        if(next == next_recored_pointer) next = NULL;
    }
    write_internal_reg(CHANNEL0_DMA_BYTE_COUNT + channel*4,num_of_bytes);
	temp_data = source_addr;
    write_internal_reg(CHANNEL0_DMA_SOURCE_ADDRESS + channel*4,temp_data & 0x5fffffff);
    temp_data = dest_addr;
	write_internal_reg(CHANNEL0_DMA_DESTINATION_ADDRESS + channel*4,temp_data & 0x5fffffff);
	if(next_recored_pointer != NULL)
    {
        temp_data = (UINT)next_recored_pointer;
        write_internal_reg(CHANNEL0NEXT_RECORD_POINTER + 4*channel,temp_data & 0x5fffffff);
        command = command | CHANNEL_ENABLE;
    }
    else
    {
        command = command | CHANNEL_ENABLE | NON_CHAIN_MOD;
    }
	
	write_internal_reg(CHANNEL0CONTROL + channel*4,command);
	if(return_flag) return;
	while(is_dma_activ(channel));
}

/** Function Header ***************************************************************************
***	DESCRIPTION:		Check the busy bit for a specific DMA channel.
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	11 JAN 1999
*** VERSION: 1.0    ***************************************************************************/

Boolean is_dma_activ(UINT channel)
{
 	if(channel > 3 ) return false;
 	 	
	if(read_internal_reg(CHANNEL0CONTROL + 4*channel) & 0x00004000) return true;
 	return false;
}

/** Function Header ***************************************************************************
***	DESCRIPTION:		Performs a DMA transfer at the slave, the function returns when the DMA transfer ends.
***						This function works only on the H family (including the 64115).
*** WRITTEN BY: 		Tal Gil
*** UPDATED LAST ON: 	31 JAN 1999
*** VERSION: 1.0    ***************************************************************************/

Boolean perform_dma_at_slave(UINT channel,UINT source_addr,UINT dest_addr,UINT num_of_words,UINT slot_number,UINT command
							 ,Boolean return_flag)
{
	UINT reg_value,pci_mem0_base,_c00,_c84;
    
    _c00 = read_internal_reg(PCI_0COMMAND);
    _c84 = read_internal_reg(PCI_1COMMAND);
	write_internal_reg(PCI_0COMMAND,0x0); 
    write_internal_reg(PCI_1COMMAND,0x0);

	if(channel > 3 ) return false;
	write_config_reg(0,PCI_0STATUS_AND_COMMAND,0,MASTER_ENABLE); /* Master enable  */
	/* Enable seconed board as a master too.    */
    write_config_reg(0,PCI_0STATUS_AND_COMMAND,slot_number,read_config_reg(0,PCI_0STATUS_AND_COMMAND,slot_number) | 7); 
    reg_value = read_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP);
	pci_mem0_base = read_internal_reg(PCI_0MEMORY0_LOW_DECODE_ADDRESS);

#if defined(GT64120) || defined(GT64120A) || defined(GT64130) || defined(GT64115)
	pci_mem0_base = pci_mem0_base << 21; /* Get PCI mem0`s base address */
#endif

	/* Remap PCI mem0 to hit the slave`s Internal registers. */
	write_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP,0xa0);
	
	/* Wait if the channel is busy  */
	while((read_word(pci_mem0_base + CHANNEL0CONTROL + channel*4) & 0x00004000) == 0x00004000);
	
	/* Prepare the transfer */
	write_word(pci_mem0_base + CHANNEL0_DMA_BYTE_COUNT + channel*4,num_of_words*4);
	write_word(pci_mem0_base + CHANNEL0_DMA_SOURCE_ADDRESS + channel*4,source_addr);
	write_word(pci_mem0_base + CHANNEL0_DMA_DESTINATION_ADDRESS + channel*4,dest_addr);
	write_word(pci_mem0_base + CHANNEL0NEXT_RECORD_POINTER + channel*4,NULL);
	
	/* Perform the transfer */
#if defined(GT64120) || defined(GT64120A)
	command = command | CHANNEL_ENABLE | NON_CHAIN_MOD; 
	write_word(pci_mem0_base + CHANNEL0CONTROL + channel*4,command);
	write_internal_reg(PCI_0COMMAND,_c00); 
    write_internal_reg(PCI_1COMMAND,_c84); 
    if(return_flag) 
	{
		write_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP,reg_value);
		return;
	}
	/* Wait for the transfer to end */
	while((read_word(pci_mem0_base + CHANNEL0CONTROL + channel*4) & 0x00004000) == 0x00004000);
#endif

#ifdef GT64130
	command = command | CHANNEL_ENABLE | NON_CHAIN_MOD; 
	write_word(pci_mem0_base + CHANNEL0CONTROL + channel*4,command);
	if(return_flag) 
	{
		write_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP,reg_value);
		return;
	}
	/* Wait for the transfer to end */
	while((read_word(pci_mem0_base + CHANNEL0CONTROL + channel*4) & 0x00004000) == 0x00004000);
#endif

#ifdef GT64115
	command = command | CHANNEL_ENABLE | NON_CHAIN_MOD; 
	write_word(pci_mem0_base + CHANNEL0CONTROL + channel*4,command);
    if(return_flag) 
	{
		write_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP,reg_value);
		return;
	}
	/* Wait for the transfer to end */
	while((read_word(pci_mem0_base + CHANNEL0CONTROL + channel*4) & 0x00004000) == 0x00004000);
#endif
	
	/* Restore the PCI mem0 remap register  */
	write_internal_reg(PCI_0MEMORY0_ADDRESS_REMAP,reg_value);
	return true;
}
	
